import 'dart:collection';

import 'package:flutter/widgets.dart';

typedef Disposer = void Function();
typedef YuroStateUpdate = void Function();

class ListNotifier implements Listenable {
  var _updaters = <YuroStateUpdate>[];
  var _groupUpdaters = HashMap<String, List<YuroStateUpdate>>();

  @override
  Disposer addListener(YuroStateUpdate listener) {
    assert(_debugAssertNotDisposed());
    _updaters.add(listener);
    return () => _updaters.remove(listener);
  }

  Disposer addListenerWithId(String id, YuroStateUpdate listener) {
    assert(_debugAssertNotDisposed());
    _groupUpdaters[id] ??= <YuroStateUpdate>[];
    _groupUpdaters[id].add(listener);
    return () => _groupUpdaters[id].remove(listener);
  }

  @override
  void removeListener(YuroStateUpdate listener) {
    assert(_debugAssertNotDisposed());
    _updaters.remove(listener);
  }

  void removeListenerWithId(String id, YuroStateUpdate listener) {
    assert(_debugAssertNotDisposed());
    if (_groupUpdaters.containsKey(id)) {
      _groupUpdaters[id].remove(listener);
    }
  }

  bool _debugAssertNotDisposed() {
    assert(() {
      if (_updaters == null) throw FlutterError('$runtimeType在调用了dispose()方法后再次被使用.');
      return true;
    }());
    return true;
  }

  @mustCallSuper
  void dispose() {
    _updaters = null;
    _groupUpdaters = null;
  }

  @protected
  void refresh() {
    assert(_debugAssertNotDisposed());
    _updaters.forEach((element) => element.call());
  }

  @protected
  void refreshGroup(String id) {
    assert(_debugAssertNotDisposed());
    _groupUpdaters[id]?.forEach((element) => element.call());
  }
}
