import 'package:flutter/scheduler.dart';
import 'package:flutter/widgets.dart';

import 'yuro_interface.dart';

typedef ValueUpdate<T> = T Function();

class _InternalFinalCallback<T> {
  ValueUpdate<T> _callback;

  _InternalFinalCallback({ValueUpdate<T> callback}) : _callback = callback;

  T call() => _callback.call();
}

mixin YuroLifeCycleBase {
  bool _initialized = false;

  bool get isInitialized => _initialized;

  bool _disposed = false;

  bool get isDisposed => _disposed;

  final onStart = _InternalFinalCallback<void>();

  final onDelete = _InternalFinalCallback<void>();

  void onInit() {
    Yuro.log('$runtimeType onInit.');
  }

  void onReady() {
    Yuro.log('$runtimeType onReady.');
  }

  void onClosed() {
    Yuro.log('$runtimeType onClosed.');
  }

  void $initLifeCycle() {
    _checkIfInitialized();
    onStart._callback = () {
      if (isInitialized) return;
      _initialized = true;
      onInit();
    };
    onDelete._callback = () {
      if (isDisposed) return;
      _disposed = true;
      onClosed();
    };
  }

  void _checkIfInitialized() {
    if (_initialized) {
      throw '[initLifeCycle()]方法只能在构造函数中调用一次';
    }
  }
}

abstract class YuroLifeCycle with YuroLifeCycleBase {
  YuroLifeCycle() {
    $initLifeCycle();
  }
}

abstract class DisposableInterface extends YuroLifeCycle {
  @override
  @mustCallSuper
  void onInit() {
    super.onInit();
    SchedulerBinding.instance?.addPostFrameCallback((_) => onReady());
  }

  @override
  @mustCallSuper
  void onReady() {
    super.onReady();
  }

  @override
  @mustCallSuper
  void onClosed() {
    super.onClosed();
  }
}
